package com.molin.study_class.service.Impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson2.JSON;
import com.molin.study_class.model.dto.ResponseObject;
import com.molin.study_class.model.param.*;
import com.molin.study_class.service.FileMakeService;
import com.molin.study_class.utils.CommonsMultipartFile;
import com.molin.study_class.utils.ExcelUtils;
import com.molin.study_class.utils.QiniuFileUtil;
import com.molin.study_class.utils.execl.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * @author David
 * @date 2025年06月07日
 * @description:
 */
@Service
@Slf4j
public class FileMakeServiceImpl implements FileMakeService {

    @Autowired
    private QiniuFileUtil qiniuFileUtil;

    @Override
    public ResponseObject createExcel(MeetAllParam meetAllParam) {
        Integer code = meetAllParam.getCode();
        List<MeetParam> data = meetAllParam.getData();
        if (code == null || code != 1 || data == null || data.isEmpty()) {
            return ResponseObject.fail("参数错误！");
        }

        MultipartFile file = null;
        try {
            file = ExcelUtils.createMeetExcelFile(meetAllParam);
        } catch (IOException e) {
            log.info("生成表格文件是吧，错误原因为：" + e.toString());
            return ResponseObject.fail("生成表格文件失败！");
        }
        if (file == null) {
            return ResponseObject.fail("生成表格文件失败！文件为空！");
        }
        try {
            JSONObject str = qiniuFileUtil.saveFile(file);
            String url = str.getString("url");
            String ossname = str.getString("name");
            return ResponseObject.success("上传成功！", url);
        } catch (IOException e) {
            log.info("上传文件到七牛云是吧，错误原因为：" + e.toString());
            return ResponseObject.fail("上传文件到七牛云失败！");
        }
    }

    @Override
    public ResponseObject createCash(CashParam param) {

        MultipartFile file = null;
        try {
            file = cashCreate(JSONObject.toJSONString(param));
        } catch (Exception e) {
            log.info("生成表格文件是吧，错误原因为：" + e.toString());
            return ResponseObject.fail("生成表格文件失败！");
        }
        if (file == null) {
            return ResponseObject.fail("生成表格文件失败！文件为空！");
        }
        try {
            JSONObject str = qiniuFileUtil.saveFile(file);
            String url = str.getString("url");
            String ossname = str.getString("name");
            return ResponseObject.success("上传成功！", url);
        } catch (IOException e) {
            log.info("上传文件到七牛云失败，错误原因为：" + e.toString());
            return ResponseObject.fail("上传文件到七牛云失败！");
        }
    }

    @Override
    public ResponseObject createLiability(LiabilityParam param) {
        MultipartFile file = null;
        try {
            file = liabilityCreate(JSONObject.toJSONString(param));
        } catch (Exception e) {
            log.info("生成表格文件是吧，错误原因为：" + e.toString());
            return ResponseObject.fail("生成表格文件失败！");
        }
        if (file == null) {
            return ResponseObject.fail("生成表格文件失败！文件为空！");
        }
        try {
            JSONObject str = qiniuFileUtil.saveFile(file);
            String url = str.getString("url");
            String ossname = str.getString("name");
            return ResponseObject.success("上传成功！", url);
        } catch (IOException e) {
            log.info("上传文件到七牛云失败，错误原因为：" + e.toString());
            return ResponseObject.fail("上传文件到七牛云失败！");
        }
    }

    @Override
    public ResponseObject createProfit(ProfitParam param) {

        MultipartFile file = null;
        try {
            file = profitCreate(JSONObject.toJSONString(param));
        } catch (Exception e) {
            log.info("生成表格文件是吧，错误原因为：" + e.toString());
            return ResponseObject.fail("生成表格文件失败！");
        }
        if (file == null) {
            return ResponseObject.fail("生成表格文件失败！文件为空！");
        }
        try {
            JSONObject str = qiniuFileUtil.saveFile(file);
            String url = str.getString("url");
            String ossname = str.getString("name");
            return ResponseObject.success("上传成功！", url);
        } catch (IOException e) {
            log.info("上传文件到七牛云失败，错误原因为：" + e.toString());
            return ResponseObject.fail("上传文件到七牛云失败！");
        }
    }


    public MultipartFile profitCreate(String json) throws Exception {
        // 1. 创建字节数组输出流

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

        // 1. JSON转Java对象
        ProfitParam data = JSON.parseObject(json, ProfitParam.class);
        ProfitParam.ProfitStatementTable table = data.getTable();
        List<ProfitParam.ProfitRow> rows = table.getRows();


        // 3. 构建数据行
//        List<List<Object>> dataList = new ArrayList<>();
//        for (ProfitParam.ProfitRow row : table.getRows()) {
//            dataList.add(Arrays.asList(
//                    row.getItem(),
//                    row.getRowNum(),
//                    row.getCurrentYearAmount(),
//                    row.getPreviousYearAmount()
//            ));
//        }
//        String desktopPath = "D:\\meet_new.xlsx";  将outputStream替换为 desktopPath，可保存于本地

        // 计算数据结束行索引
        int lastRowIndex = 4 + rows.size() - 1; // 表头结束行(索引3) + 数据行数 - 1

        // 4. 执行EasyExcel导出
        ExcelWriter excelWriter = EasyExcel.write(outputStream)
                .build();
        /*
         * 在EasyExcel中，处理器的执行顺序与注册顺序相反（先注册的先执行）。
         * 所以，我们需要确保合并处理器在边框处理器之前执行，
         * 这样边框处理器设置的样式才能覆盖合并后的单元格。
         * 下面各个handler位置不可变
         */
        WriteSheet headerSheet = EasyExcel.writerSheet(String.format(data.getFileName()+"%s年", data.getLocalYear()))
                .registerWriteHandler(new MergeHeaderHandler(0, 0, 0, 4))
                .registerWriteHandler(new MergeHeaderHandler(1, 1, 0, 1))
                .registerWriteHandler(new MergeHeaderHandler(2, 2, 0, 1))
                .registerWriteHandler(new MergeHeaderHandler(3, 3, 3, 4))
                .registerWriteHandler(new StyleHandler(0, 0, 14))// 表头专用样式
                .registerWriteHandler(new ColorHandler(1, 3, IndexedColors.GREY_25_PERCENT))
                .registerWriteHandler(new MergeListHandler(4, lastRowIndex, 3, 4))// 数据排
                .registerWriteHandler(new StyleHandler(1, 1000,  null))
                .registerWriteHandler(new BorderHandler(1, 1000))
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
//                .head(headList) // 动态表头
                .build();

        List<String> dateRow1 = Arrays.asList(String.format(data.getFileName()+"%s年", data.getLocalYear()), "", "", "", "");
        excelWriter.write(Collections.singletonList(dateRow1), headerSheet);
//        excelWriter.write(Collections.singletonList(""), headerSheet); // 空行

        // https://blog.csdn.net/qq_43980434/article/details/146021126
        // 第二部分:表格数据
        List<String> dateRow2 = Arrays.asList("纳税人识别号", "", table.getTaxpayerId(), "纳税人名称:", table.getTaxpayerName());
        excelWriter.write(Collections.singletonList(dateRow2), headerSheet);
        // 第二部分:表格数据
        List<String> dateRow3 = Arrays.asList("所属期起", "", table.getPeriodStart(), "所属期止", table.getPeriodEnd());
        excelWriter.write(Collections.singletonList(dateRow3), headerSheet);
        List<String> dateRow4 = Arrays.asList("项目", "行次", "本年累计金额", "上年金额", "");
        excelWriter.write(Collections.singletonList(dateRow4), headerSheet);

        //动态数据：这个我们可以将数据库查出的对象动态写入
        excelWriter.write(rows, headerSheet);
        excelWriter.finish();

        // 第三部分：尾部
//        List<String> dateRow4 = Arrays.asList("金额大写", "肆佰元整", "", "");
//        excelWriter.write(Collections.singletonList(dateRow4), headerSheet);
//        excelWriter.write(Collections.singletonList(""), headerSheet); // 空行


        // 转换为字节数组
        byte[] excelBytes = outputStream.toByteArray();

        // 创建自定义 MultipartFile 实现
        return new CommonsMultipartFile(
                new ByteArrayResource(excelBytes),
                URLEncoder.encode(data.getFileName() + ".xlsx", "UTF-8"),
                "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
        );
    }


    public MultipartFile cashCreate(String json) throws Exception {
        // 1. 创建字节数组输出流

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

        // 1. JSON转Java对象
        CashParam data = JSON.parseObject(json, CashParam.class);
        CashParam.CashTable table = data.getTable();
        List<CashParam.CashRow> rows = table.getRows();


        // 3. 构建数据行
//        List<List<Object>> dataList = new ArrayList<>();
//        for (ProfitParam.ProfitRow row : table.getRows()) {
//            dataList.add(Arrays.asList(
//                    row.getItem(),
//                    row.getRowNum(),
//                    row.getCurrentYearAmount(),
//                    row.getPreviousYearAmount()
//            ));
//        }
//        String desktopPath = "D:\\meet_new.xlsx";  将outputStream替换为 desktopPath，可保存于本地

        // 计算数据结束行索引
        int lastRowIndex = 4 + rows.size() - 1; // 表头结束行(索引3) + 数据行数 - 1

        // 4. 执行EasyExcel导出
        ExcelWriter excelWriter = EasyExcel.write(outputStream)
                .build();
        /*
         * 在EasyExcel中，处理器的执行顺序与注册顺序相反（先注册的先执行）。
         * 所以，我们需要确保合并处理器在边框处理器之前执行，
         * 这样边框处理器设置的样式才能覆盖合并后的单元格。
         * 下面各个handler位置不可变
         */
        WriteSheet headerSheet = EasyExcel.writerSheet(String.format(data.getFileName()+"%s年", data.getLocalYear()))
                .registerWriteHandler(new MergeHeaderHandler(0, 0, 0, 4))
                .registerWriteHandler(new MergeHeaderHandler(1, 1, 0, 1))
                .registerWriteHandler(new MergeHeaderHandler(2, 2, 0, 1))
                .registerWriteHandler(new MergeHeaderHandler(3, 3, 3, 4))
                .registerWriteHandler(new StyleHandler(0, 0, 14))// 表头专用样式
                .registerWriteHandler(new ColorHandler(1, 3, IndexedColors.GREY_25_PERCENT))
                .registerWriteHandler(new MergeListHandler(4, lastRowIndex, 3, 4))// 数据排
                .registerWriteHandler(new StyleHandler(1, 1000,  null))
                .registerWriteHandler(new BorderHandler(1, 1000))
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
//                .head(headList) // 动态表头
                .build();

        List<String> dateRow1 = Arrays.asList(String.format(data.getFileName()+"%s年", data.getLocalYear()), "", "", "", "");
        excelWriter.write(Collections.singletonList(dateRow1), headerSheet);
//        excelWriter.write(Collections.singletonList(""), headerSheet); // 空行

        // https://blog.csdn.net/qq_43980434/article/details/146021126
        // 第二部分:表格数据
        List<String> dateRow2 = Arrays.asList("纳税人识别号", "", table.getTaxpayerId(), "纳税人名称:", table.getTaxpayerName());
        excelWriter.write(Collections.singletonList(dateRow2), headerSheet);
        // 第二部分:表格数据
        List<String> dateRow3 = Arrays.asList("所属期起", "", table.getPeriodStart(), "所属期止", table.getPeriodEnd());
        excelWriter.write(Collections.singletonList(dateRow3), headerSheet);
        List<String> dateRow4 = Arrays.asList("项目", "行次", "本年累计金额", "上年金额", "");
        excelWriter.write(Collections.singletonList(dateRow4), headerSheet);

        //动态数据：这个我们可以将数据库查出的对象动态写入
        excelWriter.write(rows, headerSheet);
        excelWriter.finish();

        // 第三部分：尾部
//        List<String> dateRow4 = Arrays.asList("金额大写", "肆佰元整", "", "");
//        excelWriter.write(Collections.singletonList(dateRow4), headerSheet);
//        excelWriter.write(Collections.singletonList(""), headerSheet); // 空行


        // 转换为字节数组
        byte[] excelBytes = outputStream.toByteArray();

        // 创建自定义 MultipartFile 实现
        return new CommonsMultipartFile(
                new ByteArrayResource(excelBytes),
                URLEncoder.encode(data.getFileName() + ".xlsx", "UTF-8"),
                "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
        );
    }


    public MultipartFile liabilityCreate(String json) throws Exception {
        // 1. 创建字节数组输出流
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

        // 1. JSON转Java对象
        LiabilityParam data = JSON.parseObject(json, LiabilityParam.class);
        LiabilityParam.LiabilityTable table = data.getTable();
        List<LiabilityParam.LiabilityRow> rows = table.getRows();


        // 3. 构建数据行
//        List<List<Object>> dataList = new ArrayList<>();
//        for (ProfitParam.ProfitRow row : table.getRows()) {
//            dataList.add(Arrays.asList(
//                    row.getItem(),
//                    row.getRowNum(),
//                    row.getCurrentYearAmount(),
//                    row.getPreviousYearAmount()
//            ));
//        }
//        String desktopPath = "D:\\meet_new.xlsx";  将outputStream替换为 desktopPath，可保存于本地

        // 计算数据结束行索引
        int lastRowIndex = 4 + rows.size() - 1; // 表头结束行(索引3) + 数据行数 - 1

        // 4. 执行EasyExcel导出
        ExcelWriter excelWriter = EasyExcel.write(outputStream)
                .build();
        /*
         * 在EasyExcel中，处理器的执行顺序与注册顺序相反（先注册的先执行）。
         * 所以，我们需要确保合并处理器在边框处理器之前执行，
         * 这样边框处理器设置的样式才能覆盖合并后的单元格。
         * 下面各个handler位置不可变
         */
        WriteSheet headerSheet = EasyExcel.writerSheet(String.format(data.getFileName()+"%s年", data.getLocalYear()))
                .registerWriteHandler(new MergeHeaderHandler(0, 0, 0, 7))

                .registerWriteHandler(new MergeHeaderHandler(1, 1, 0, 1))
                .registerWriteHandler(new MergeHeaderHandler(1, 1, 2, 3))
                .registerWriteHandler(new MergeHeaderHandler(1, 1, 4, 5))
                .registerWriteHandler(new MergeHeaderHandler(1, 1, 6, 7))

                .registerWriteHandler(new MergeHeaderHandler(2, 2, 0, 1))
                .registerWriteHandler(new MergeHeaderHandler(2, 2, 2, 3))
                .registerWriteHandler(new MergeHeaderHandler(2, 2, 4, 5))
                .registerWriteHandler(new MergeHeaderHandler(2, 2, 6, 7))

                .registerWriteHandler(new StyleHandler(0, 0, 14))// 表头专用样式
                .registerWriteHandler(new ColorHandler(1, 3, IndexedColors.GREY_25_PERCENT))
//                .registerWriteHandler(new MergeListHandler(4, lastRowIndex, 3, 4))// 数据排
                .registerWriteHandler(new StyleHandler(1, 1000,  null))
                .registerWriteHandler(new BorderHandler(1, 1000))
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy())
//                .head(headList) // 动态表头
                .build();

        List<String> dateRow1 = Arrays.asList(String.format(data.getFileName()+"%s年", data.getLocalYear()), "", "", "", "");
        excelWriter.write(Collections.singletonList(dateRow1), headerSheet);
//        excelWriter.write(Collections.singletonList(""), headerSheet); // 空行

        // https://blog.csdn.net/qq_43980434/article/details/146021126
        // 第二部分:表格数据
        List<String> dateRow2 = Arrays.asList("纳税人识别号", "", table.getTaxpayerId(),"", "纳税人名称:", "", table.getTaxpayerName(),"");
        excelWriter.write(Collections.singletonList(dateRow2), headerSheet);
        // 第二部分:表格数据
        List<String> dateRow3 = Arrays.asList("所属期起", "", table.getPeriodStart(),"", "所属期止","", table.getPeriodEnd(),"");
        excelWriter.write(Collections.singletonList(dateRow3), headerSheet);
        List<String> dateRow4 = Arrays.asList("资产", "行次", "期末余额", "年初余额", "负债和所有者权益","行次","期末余额","年初余额");
        excelWriter.write(Collections.singletonList(dateRow4), headerSheet);

        //动态数据：这个我们可以将数据库查出的对象动态写入
        excelWriter.write(rows, headerSheet);
        excelWriter.finish();

        // 第三部分：尾部
//        List<String> dateRow4 = Arrays.asList("金额大写", "肆佰元整", "", "");
//        excelWriter.write(Collections.singletonList(dateRow4), headerSheet);
//        excelWriter.write(Collections.singletonList(""), headerSheet); // 空行


        // 转换为字节数组
        byte[] excelBytes = outputStream.toByteArray();

        // 创建自定义 MultipartFile 实现
        return new CommonsMultipartFile(
                new ByteArrayResource(excelBytes),
                URLEncoder.encode(data.getFileName() + ".xlsx", "UTF-8"),
                "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
        );
    }


}
