package com.molin.study_class.utils;


import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.qiniu.common.QiniuException;
import com.qiniu.common.Zone;
import com.qiniu.http.Response;
import com.qiniu.storage.BucketManager;
import com.qiniu.storage.Configuration;
import com.qiniu.storage.UploadManager;
import com.qiniu.storage.model.DefaultPutRet;
import com.qiniu.util.Auth;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.Arrays;
import java.util.UUID;

/**
 * @ClassName UploadFileUtil
 * @Description: 七牛云工具类
 * @Author gzm
 * @Date 2020/5/14 16:54
 **/
@Component
// @ConfigurationProperties(prefix = "zlt.study.oss.qiniu")
public class QiniuFileUtil {

    @Value("${zlt.elp.oss.qiniu.AccessKey}")
    private String accessKey;

    @Value("${zlt.elp.oss.qiniu.SecretKey}")
    private String secretKey;

    @Value("${zlt.elp.oss.qiniu.Bucket}")
    private String bucket;

    @Value("${zlt.elp.oss.qiniu.BucketUrl}")
    private String bucketUrl;

    /**
     * 在七牛云上保存文件
     * 返回字符串
     *
     * @param file
     * @return
     */

    public JSONObject saveFile(MultipartFile file) throws IOException {
        JSONObject retObj = new JSONObject();
        //构造一个带指定Region对象的配置类
        Configuration cfg = new Configuration(Zone.zone2());
        //生成上传凭证，然后准备上传
        UploadManager uploadManager = new UploadManager(cfg);
        //创建权限类似于token
        Auth auth = Auth.create(accessKey, secretKey);
        //生成一个上传的token
        String upToken = auth.uploadToken(bucket);

        int pos = file.getOriginalFilename().lastIndexOf(".");
        if (pos < 0) {
            return null;
        }
        //生存文件名
        String fileExt = file.getOriginalFilename().substring(pos + 1).toLowerCase();
        String fileName = UUID.randomUUID().toString().replaceAll("-", "") + "." + fileExt;
        // 开始上传
        Response response = uploadManager.put(file.getBytes(), fileName, upToken);
        //解析上传成功的结果
        DefaultPutRet putRet = new Gson().fromJson(response.bodyString(), DefaultPutRet.class);
        retObj.put("url", "https://" + bucketUrl + "/" + putRet.key);
        retObj.put("name", putRet.key);
        retObj.put("hash", putRet.hash);
        return retObj;
    }

    public String getFileSuffix(String fileName) {
        //获取文件后缀名
        return fileName.substring(fileName.lastIndexOf(".") + 1);
    }

    private static final String[] allowSuffix = new String[]{".bmp", ".jpg", ".jpeg", "gif", ".png"};

    public Boolean checkSuffix(MultipartFile file) {
        String suffix = getFileSuffix(file.getOriginalFilename());
        if (!Arrays.asList(allowSuffix).contains(suffix)) {
            return false;
        }
        return true;
    }

    public boolean deleteFileFromQiniu(String fileName) {
        //构造一个带指定Zone对象的配置类
        Configuration cfg = new Configuration(Zone.zone2());
        String key = fileName;
        Auth auth = Auth.create(accessKey, secretKey);
        BucketManager bucketManager = new BucketManager(auth, cfg);
        try {
            Response delete = bucketManager.delete(bucket, key);
            return delete.statusCode > 0;
        } catch (QiniuException ex) {
            //如果遇到异常，说明删除失败
            ex.printStackTrace();
            return false;
        }
    }
}
