package com.molin.study_class.utils.execl;

import com.alibaba.excel.write.handler.CellWriteHandler;
import com.alibaba.excel.write.handler.context.CellWriteHandlerContext;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.RegionUtil;

/**
 * @author David
 * @date 2025年06月12日
 * @description:
 */
//加边框
public class BorderHandler implements CellWriteHandler {
    private int startRow;
    private int endRow;

    public BorderHandler(int startRow, int endRow){
        this.startRow = startRow;
        this.endRow = endRow;
    }
    @Override
    public void afterCellDispose(CellWriteHandlerContext context) {
        int startRow = this.startRow;
        int endRow = this.endRow; // 根据需要调整

        // 获取当前单元格
        Cell cell = context.getCell();
        int rowIndex = cell.getRowIndex();

        // 如果单元格在动态数据范围内，添加边框
        if (rowIndex >= startRow && rowIndex <= endRow) {
            Workbook workbook = cell.getSheet().getWorkbook();
            Sheet sheet = cell.getSheet();
            // 检查单元格是否在合并区域内
            CellRangeAddress mergedRegion = getMergedRegion(sheet, rowIndex, cell.getColumnIndex());
            if (mergedRegion != null) {
                // 如果是合并区域，设置整个区域的边框，
                setMergedBorders(sheet, mergedRegion);
            } else {
                // 普通单元格设置边框
                setCellBorders(cell);
            }
        }
    }

    // 获取单元格所在的合并区域
    private CellRangeAddress getMergedRegion(Sheet sheet, int row, int column) {
        for (int i = 0; i < sheet.getNumMergedRegions(); i++) {
            CellRangeAddress region = sheet.getMergedRegion(i);
            if (region.isInRange(row, column)) {
                return region;
            }
        }
        return null;
    }

    // 为合并区域设置边框
    private void setMergedBorders(Sheet sheet, CellRangeAddress region) {
        RegionUtil.setBorderTop(BorderStyle.THIN, region, sheet);
        RegionUtil.setBorderBottom(BorderStyle.THIN, region, sheet);
        RegionUtil.setBorderLeft(BorderStyle.THIN, region, sheet);
        RegionUtil.setBorderRight(BorderStyle.THIN, region, sheet);

        RegionUtil.setTopBorderColor(IndexedColors.BLACK.getIndex(), region, sheet);
        RegionUtil.setBottomBorderColor(IndexedColors.BLACK.getIndex(), region, sheet);
        RegionUtil.setLeftBorderColor(IndexedColors.BLACK.getIndex(), region, sheet);
        RegionUtil.setRightBorderColor(IndexedColors.BLACK.getIndex(), region, sheet);
    }

    // 为普通单元格设置边框
    private void setCellBorders(Cell cell) {
        Workbook workbook = cell.getSheet().getWorkbook();
        CellStyle newStyle = workbook.createCellStyle();
        CellStyle existingStyle = cell.getCellStyle();

        if (existingStyle != null) {
            newStyle.cloneStyleFrom(existingStyle);
        }

        newStyle.setBorderTop(BorderStyle.THIN);
        newStyle.setBorderBottom(BorderStyle.THIN);
        newStyle.setBorderLeft(BorderStyle.THIN);
        newStyle.setBorderRight(BorderStyle.THIN);
        newStyle.setTopBorderColor(IndexedColors.BLACK.getIndex());
        newStyle.setBottomBorderColor(IndexedColors.BLACK.getIndex());
        newStyle.setLeftBorderColor(IndexedColors.BLACK.getIndex());
        newStyle.setRightBorderColor(IndexedColors.BLACK.getIndex());

        cell.setCellStyle(newStyle);
    }


}
