package com.molin.study_class.utils.execl;

import com.alibaba.excel.write.handler.CellWriteHandler;
import com.alibaba.excel.write.handler.context.CellWriteHandlerContext;
import org.apache.poi.ss.usermodel.*;

/**
 * @author David
 * @date 2025年06月12日
 * @description:  自定义颜色处理器
 */
public class ColorHandler  implements CellWriteHandler {


    private final int startRow;
    private final int endRow;
    private final IndexedColors color;

    public ColorHandler(int startRow, int endRow, IndexedColors color) {
        this.startRow = startRow;
        this.endRow = endRow;
        this.color = color;
    }

    @Override
    public void afterCellCreate(CellWriteHandlerContext context) {
        Cell cell = context.getCell();
        Row row = cell.getRow();
        int rowIndex = row.getRowNum();

        // 检查是否在目标行范围内
        if (rowIndex >= startRow && rowIndex <= endRow) {
            Workbook workbook = context.getWriteSheetHolder().getSheet().getWorkbook();
            CellStyle newCellStyle = workbook.createCellStyle();
            CellStyle oldStyle = cell.getCellStyle();
            if(oldStyle != null){
                // 复制原有样式避免覆盖
                newCellStyle.cloneStyleFrom(cell.getCellStyle());
            }
            // 设置背景色
            newCellStyle.setFillForegroundColor(color.getIndex());
            newCellStyle.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            cell.setCellStyle(newCellStyle);
        }
    }






}
