package com.molin.study_class.utils.execl;

/**
 * @author David
 * @date 2025年06月12日
 * @description:
 */

import com.alibaba.excel.metadata.Head;
import com.alibaba.excel.metadata.data.WriteCellData;
import com.alibaba.excel.write.handler.CellWriteHandler;
import com.alibaba.excel.write.metadata.holder.WriteSheetHolder;
import com.alibaba.excel.write.metadata.holder.WriteTableHolder;
import org.apache.poi.ss.usermodel.BorderStyle;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;
import org.apache.poi.ss.util.RegionUtil;

import java.util.List;

/**
 * @author David
 * @date 2025年06月12日
 * @description: 单元格合并器--动态数据列的合并（非表头，多用与未知数据行数中的合并）
 */
public class MergeListHandler implements CellWriteHandler {
    private final int startRow;
    private final int endRow;
    private final int startCol;
    private final int endCol;

    public MergeListHandler(int startRow, int endRow, int startCol, int endCol) {
        this.startRow = startRow;
        this.endRow = endRow;
        this.startCol = startCol;
        this.endCol = endCol;
    }


    // 只处理指定范围内的行
    @Override
    public void afterCellDispose(WriteSheetHolder writeSheetHolder,
                                 WriteTableHolder writeTableHolder,
                                 List<WriteCellData<?>> cellDataList,
                                 Cell cell,
                                 Head head,
                                 Integer relativeRowIndex,
                                 Boolean isHead) {
        // 只处理指定范围内的行
        // 只处理数据行（非表头）
        if (isHead != null && !isHead && relativeRowIndex != null) {
            Sheet sheet = writeSheetHolder.getSheet();
            int absoluteRowIndex = startRow + relativeRowIndex;

            if (absoluteRowIndex >= startRow && absoluteRowIndex <= endRow) {
                // 检查是否已经合并过
                boolean alreadyMerged = false;
                for (int i = 0; i < sheet.getNumMergedRegions(); i++) {
                    CellRangeAddress region = sheet.getMergedRegion(i);
                    if (region.getFirstRow() == absoluteRowIndex &&
                            region.getFirstColumn() == startCol) {
                        alreadyMerged = true;
                        break;
                    }
                }

                if (!alreadyMerged) {
                    // 添加合并区域
                    CellRangeAddress mergeRegion = new CellRangeAddress(
                            absoluteRowIndex,
                            absoluteRowIndex,
                            startCol,
                            endCol
                    );
                    sheet.addMergedRegion(mergeRegion);
//                    // 设置合并单元格的边框（不可在这立即处理合并后的单元格样式）
//                    setMergedRegionBorders(sheet, mergeRegion);
                }
            }
        }
    }

    /**
     * 为合并区域设置边框, BorderCellStyleHandler 也可处理这种合并后的单元框
     * {@link BorderHandler}
     */
    private void setMergedRegionBorders(Sheet sheet, CellRangeAddress region) {
        // 使用RegionUtil设置四周边框
        RegionUtil.setBorderTop(BorderStyle.THIN, region, sheet);
        RegionUtil.setBorderBottom(BorderStyle.THIN, region, sheet);
        RegionUtil.setBorderLeft(BorderStyle.THIN, region, sheet);
        RegionUtil.setBorderRight(BorderStyle.THIN, region, sheet);

        // 设置边框颜色
        RegionUtil.setTopBorderColor(IndexedColors.BLACK.getIndex(), region, sheet);
        RegionUtil.setBottomBorderColor(IndexedColors.BLACK.getIndex(), region, sheet);
        RegionUtil.setLeftBorderColor(IndexedColors.BLACK.getIndex(), region, sheet);
        RegionUtil.setRightBorderColor(IndexedColors.BLACK.getIndex(), region, sheet);
    }

}
