package com.molin.study_class.utils.execl;

import com.alibaba.excel.write.handler.CellWriteHandler;
import com.alibaba.excel.write.handler.context.CellWriteHandlerContext;
import org.apache.poi.ss.usermodel.*;

/**
 * @author David
 * @date 2025年06月12日
 * @description:
 */
// 设置单元格样式：水平和垂直居中 处理器
public class StyleHandler implements CellWriteHandler {

    private final int startRow;
    private final int endRow;
    private Integer fontSize;


    public StyleHandler(int startRow, int endRow, Integer fontSize) {
        this.startRow = startRow;
        this.endRow = endRow;
        this.fontSize = fontSize;
    }

    @Override
    public void afterCellDispose(CellWriteHandlerContext context) {
        // 获取单元格
        Cell cell = context.getCell();
        // 获取单元格所在的行对象
        Row row = cell.getRow();
        // 获取行的索引
        int rowIndex = row.getRowNum();

        // 检查是否在目标行范围内
        if (rowIndex >= startRow && rowIndex <= endRow) {
            Workbook workbook = context.getWriteSheetHolder().getSheet().getWorkbook();
            CellStyle newCellStyle = workbook.createCellStyle();
            CellStyle oldStyle = cell.getCellStyle();
            if(oldStyle != null){
                // 复制原有样式避免覆盖
                newCellStyle.cloneStyleFrom(cell.getCellStyle());
            }
            // 设置字体样式
            newCellStyle.setAlignment(HorizontalAlignment.CENTER);
            newCellStyle.setVerticalAlignment(VerticalAlignment.CENTER);
            if(null != fontSize && fontSize > 0){
                Font font = workbook.createFont();
                short size = Short.parseShort(String.valueOf(fontSize));
                font.setFontHeightInPoints(size);
//                font.setFontName("宋体"); // 默认中文字体
                newCellStyle.setFont(font);
            }
            cell.setCellStyle(newCellStyle);
        }
    }
}
